/**
 * SelectableNode.java
 *
 * Copyright (c) 2011-2014, JFXtras All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer. * Redistributions in binary
 * form must reproduce the above copyright notice, this list of conditions and
 * the following disclaimer in the documentation and/or other materials provided
 * with the distribution. * Neither the name of the organization nor the names
 * of its contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package jfxtras.scene.control.window;

import javafx.beans.property.ReadOnlyBooleanProperty;

/**
 * A node must implement this interface to be selectable. Usually, nodes/windows
 * are selected via selection rectangle gesture.
 *
 * @author Michael Hoffer &lt;info@michaelhoffer.de&gt;
 *
 */
public interface SelectableNode {

    /**
     * Requests selection/deselection.
     *
     * @param select defines whether to select or deselect the node
     * @return <code>true</code> if request is accepted;<code>false</code>
     * otherwise
     */
    public boolean requestSelection(boolean select);

    /**
     * Indicates whether this node is selected.
     *
     * @return {@code true} if this node is selected; {@code false} otherwise
     */
    public boolean isSelected();

    /**
     * Indicates whether this node is selected.
     *
     * @return {@code true} if this node is selected; {@code false} otherwise
     */
    public ReadOnlyBooleanProperty selectedProperty();
}
